package com.daim.flutter_apm

import com.moonmiles.apmservices.model.abs.APMModel
import com.moonmiles.apmservices.net.APMException
import com.moonmiles.apmservices.utils.APMReactUtils
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject


object APMFlutterUtils {
    fun resolveResult(obj: Any?, result: Result) {
        try {
            if (obj != null) {
                if (obj is ArrayList<*>) {
                    resolveResultWithArray(obj as ArrayList<Any>, result)
                }
                else if (obj is APMModel) {
                    resolveResultWithObject(obj as APMModel, result)
                }
                else {
                    result.success(obj)
                }
            }
            else {
                result.success(null)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            rejectResult(e, result)
        }
    }

    @kotlin.jvm.Throws(JSONException::class)
    private fun resolveResultWithObject(obj: APMModel, result: Result) {
        val params: HashMap<String, Any>? = convertObjectToMap(obj)
        result.success(params)
    }

    @kotlin.jvm.Throws(JSONException::class)
    private fun resolveResultWithArray(obj: ArrayList<Any>, result: Result) {
        val array: ArrayList<Any>? = convertObjectToArray(obj)
        result.success(array)
    }

    //Reject
    //********************//
    fun rejectResult(obj: Any?, result: Result) {
        try {
            if (obj != null) {
                if (obj is APMException) {
                    rejectResultWithAPMException(obj, result)
                }
                else if (obj is java.lang.Exception) {
                    rejectResultWithException(obj, result)
                }
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            rejectResultWithException(e, result)
        }
    }

    private fun rejectResultWithException(e: java.lang.Exception, result: Result) {
        val exception = APMException(500, e.localizedMessage)
        rejectResultWithAPMException(exception, result)
    }

    private fun rejectResultWithAPMException(e: APMException, result: Result) {
        result.error(e.errorCode.toString() + "", e.errorMessage, e.errorMessage)
    }

    //Convert
    //********************//
    @Throws(JSONException::class)
    fun convertObjectToMap(obj: APMModel): HashMap<String, Any>? {
        var params: HashMap<String, Any>? = null
        val jsonObject = obj.toJSONObject()
        params = convertJsonToMap(jsonObject)
        return params
    }

    @Throws(JSONException::class)
    fun convertObjectToArray(obj: ArrayList<Any>?): ArrayList<Any>? {
        var array: ArrayList<Any>? = null
        val userjson = APMReactUtils.toJSONArray(obj)
        array = convertJsonToArray(userjson)
        return array
    }

    @Throws(JSONException::class)
    fun convertJsonToMap(jsonObject: JSONObject): HashMap<String, Any>? {
        val map: HashMap<String, Any> = HashMap()
        val iterator = jsonObject.keys()
        while (iterator.hasNext()) {
            val key = iterator.next()
            val value = jsonObject[key]
            if (value is JSONObject) {
                map.put(key, convertJsonToMap(value) as Any)
            } else if (value is JSONArray) {
                map.put(key, convertJsonToArray(value) as Any)
            } else if (value is Boolean) {
                map.put(key, value)
            } else if (value is Int) {
                map.put(key, value)
            } else if (value is Double) {
                map.put(key, value)
            } else if (value is String) {
                map.put(key, value)
            } else {
                map.put(key, value.toString())
            }
        }
        return map
    }

    @Throws(JSONException::class)
    fun convertJsonToArray(jsonArray: JSONArray): ArrayList<Any>? {
        val array = ArrayList<Any>()
        for (i in 0 until jsonArray.length()) {
            val value = jsonArray[i]
            if (value is JSONObject) {
                array.add(convertJsonToMap(value) as Any)
            } else if (value is JSONArray) {
                array.add(convertJsonToArray(value) as Any)
            } else if (value is Boolean) {
                array.add(value)
            } else if (value is Int) {
                array.add(value)
            } else if (value is Double) {
                array.add(value)
            } else if (value is String) {
                array.add(value)
            } else {
                array.add(value.toString())
            }
        }
        return array
    }
}
