package com.daim.flutter_apm

import android.util.Log
import androidx.annotation.NonNull
import com.moonmiles.apm.sdk.APMOnBadgeClickListener
import com.moonmiles.apm.sdk.APMPrivate
import com.moonmiles.apmservices.model.APMUser
import com.moonmiles.apmservices.model.local.APMTag
import com.moonmiles.apmservices.net.APMException
import com.moonmiles.apmservices.sdk.APMServicesPrivate
import com.moonmiles.apmservices.sdk.APMServicesUserListener
import com.moonmiles.apmservices.utils.APMServicesTriggerActionListener
import com.moonmiles.apmservices.utils.APMServicesTriggerActionsListener
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


/** APMModule */
class APMModule : FlutterPlugin, MethodCallHandler, APMServicesUserListener, APMOnBadgeClickListener {
    private lateinit var channel: MethodChannel

    private lateinit var servicesUserEventChannel: EventChannel
    private var servicesUserListener: Any? = null
    private var servicesUserEventSink: EventChannel.EventSink? = null

    private lateinit var onBadgeClickEventChannel: EventChannel
    private var onBadgeClickListener: Any? = null
    private var onBadgeClickEventSink: EventChannel.EventSink? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "apm_module")
        channel.setMethodCallHandler(this)

        servicesUserEventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "apm_module_services_user_event")
        servicesUserEventChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(listener: Any?, eventSink: EventChannel.EventSink?) {
                APMServicesPrivate.sharedInstancePrivate().servicesUserListener = this@APMModule
                servicesUserListener = listener
                servicesUserEventSink = eventSink
            }

            override fun onCancel(listener: Any?) {
                APMServicesPrivate.sharedInstancePrivate().servicesUserListener = null
                servicesUserListener = null
                servicesUserEventSink = null
            }
        })

        onBadgeClickEventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "apm_module_on_badge_click_event")
        onBadgeClickEventChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(listener: Any?, eventSink: EventChannel.EventSink?) {
                APMPrivate.sharedInstanceBadgeManagerPrivate().onBadgeClickListener = this@APMModule
                onBadgeClickListener = listener
                onBadgeClickEventSink = eventSink
            }

            override fun onCancel(listener: Any?) {
                APMPrivate.sharedInstanceBadgeManagerPrivate().onBadgeClickListener = null
                onBadgeClickListener = null
                onBadgeClickEventSink = null
            }
        })
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    //EVOLUTION : trouver une solution pour appeler une méthode directement via un String
    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val arguments: ArrayList<*>
        if(call.arguments != null) {
            arguments = call.arguments as ArrayList<*>
        }
        else {
            arguments = ArrayList<Any>()
        }

        if (call.method == "initAppWithPartnerID") {
            initAppWithPartnerID(arguments, result)
        }
        else if (call.method == "initAppWithAppID") {
            initAppWithAppID(arguments, result)
        }
        else if (call.method == "refreshSDK") {
            refreshSDK(arguments, result)
        }
        else if (call.method == "triggerActions") {
            triggerActions(arguments, result)
        }
        else if (call.method == "triggerActionsWithProperties") {
            triggerActionsWithProperties(arguments, result)
        }
        else if (call.method == "triggerActionsWithCompletion") {
            triggerActionsWithCompletion(arguments, result)
        }
        else if (call.method == "triggerActionsWithTags") {
            triggerActionsWithTags(arguments, result)
        }
        else if (call.method == "triggerAction") {
            triggerAction(arguments, result)
        }
        else if (call.method == "triggerActionWithProperties") {
            triggerActionWithProperties(arguments, result)
        }
        else if (call.method == "triggerActionWithCompletion") {
            triggerActionWithCompletion(arguments, result)
        }
        else if (call.method == "triggerActionWithPropertiesAndCompletion") {
            triggerActionWithPropertiesAndCompletion(arguments, result)
        }
        else if (call.method == "triggerActionWithTag") {
            triggerActionWithTag(arguments, result)
        }
        else if (call.method == "showBadge") {
            showBadge(arguments, result)
        }
        else if (call.method == "hideBadge") {
            hideBadge(arguments, result)
        }
        else if (call.method == "isOpenBigBadge") {
            isOpenBigBadge(arguments, result)
        }
        else if (call.method == "openBigBadge") {
            openBigBadge(arguments, result)
        }
        else if (call.method == "openBigBadgeWithController") {
            openBigBadgeWithController(arguments, result)
        }
        else if (call.method == "closeBigBadge") {
            closeBigBadge(arguments, result)
        }
        else if (call.method == "showOnboarding") {
            showOnboarding(arguments, result)
        }
        else if (call.method == "showOnboardingConnected") {
            showOnboardingConnected(arguments, result)
        }
        else if (call.method == "setBaseUrl") {
            setBaseUrl(arguments, result)
        }
        else if (call.method == "getBaseUrl") {
            getBaseUrl(arguments, result)
        }
        else if (call.method == "setDebugMode") {
            setDebugMode(arguments, result)
        }
        else if (call.method == "isDebugMode") {
            isDebugMode(arguments, result)
        }
        else if (call.method == "setDeviceStatus") {
            setDeviceStatus(arguments, result)
        }
        else if (call.method == "setDeviceOptIn") {
            setDeviceOptIn(arguments, result)
        }
        else if (call.method == "user") {
            user(arguments, result)
        }
        else if (call.method == "device") {
            device(arguments, result)
        }
        else if (call.method == "setBadgeEnabled") {
            setBadgeEnabled(arguments, result)
        }
        else if (call.method == "setInterstitialEnabled") {
            setInterstitialEnabled(arguments, result)
        }
        else if (call.method == "setNotificationFullscreenEnabled") {
            setNotificationFullscreenEnabled(arguments, result)
        }
        else {
            result.notImplemented()
        }
    }

    private fun initAppWithPartnerID(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().initApp(arguments[0] as String?, arguments[1] as String?)
        result.success(null)
    }

    private fun initAppWithAppID(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().initApp(arguments[0] as String?, arguments[1] as String?, arguments[2] as String?)
        result.success(null)
    }

    private fun refreshSDK(arguments: ArrayList<*>, result: Result) {
        APMServicesPrivate.sharedInstancePrivate().refreshSDK(arguments[0] as Boolean)
        result.success(null)
    }

    private fun triggerActions(arguments: ArrayList<*>, result: Result) {
        //List actionName
        val list: ArrayList<String> = arguments[0] as ArrayList<String>
        val arrayActionName: Array<String> = list.toTypedArray()

        APMPrivate.sharedInstancePrivate().triggerActions(arrayActionName)
        result.success(null)
    }

    private fun triggerActionsWithProperties(arguments: ArrayList<*>, result: Result) {
        //List actionName
        val list: ArrayList<String> = arguments[0] as ArrayList<String>
        val arrayActionName: Array<String> = list.toTypedArray()

        //List properties
        val list2: ArrayList<HashMap<String, String>> = arguments[1] as ArrayList<HashMap<String, String>>
        val arrayProperties: Array<HashMap<String, String>> = list2.toTypedArray()

        APMPrivate.sharedInstancePrivate().triggerActions(arrayActionName, arrayProperties)
        result.success(null)
    }

    private fun triggerActionsWithCompletion(arguments: ArrayList<*>, result: Result) {
        //List actionName
        val list: ArrayList<String> = arguments[0] as ArrayList<String>
        val arrayActionName: Array<String> = list.toTypedArray()

        //List properties
        val list2: ArrayList<HashMap<String, String>> = arguments[1] as ArrayList<HashMap<String, String>>
        val arrayProperties: Array<HashMap<String, String>> = list2.toTypedArray()

        APMPrivate.sharedInstancePrivate().triggerActions(arrayActionName, arrayProperties, object : APMServicesTriggerActionsListener {
            override fun completion(listStatusActionName: java.util.ArrayList<*>?, exception: APMException?) {
                APMPrivate.sharedInstancePrivate().m_handler.post(object : Runnable {
                    override fun run() {
                        if(exception != null) {
                            APMFlutterUtils.rejectResult(exception, result)
                        }
                        else {
                            APMFlutterUtils.resolveResult(listStatusActionName, result)
                        }
                    }
                })
            }
        })
    }

    private fun triggerActionsWithTags(arguments: ArrayList<*>, result: Result) {
        val listTags: ArrayList<HashMap<*, *>> = arguments[0] as ArrayList<HashMap<*, *>>
        var arrayTags: Array<APMTag?>? = null
        if (listTags != null) {
            val tags: ArrayList<APMTag> = ArrayList()
            for (i in 0 until listTags.size) {
                val mapTag: HashMap<*, *> = listTags.get(i)
                if (mapTag != null) {
                    val tag = APMTag.tagFromHashMap(mapTag)
                    tags.add(tag)
                }
            }
            arrayTags = tags.toTypedArray()
        }

        APMPrivate.sharedInstancePrivate().triggerActions(arrayTags, object : APMServicesTriggerActionsListener {
            override fun completion(listStatusActionName: java.util.ArrayList<*>?, exception: APMException?) {
                APMPrivate.sharedInstancePrivate().m_handler.post(object : Runnable {
                    override fun run() {
                        if(exception != null) {
                            APMFlutterUtils.rejectResult(exception, result)
                        }
                        else {
                            APMFlutterUtils.resolveResult(listStatusActionName, result)
                        }
                    }
                })
            }
        })
    }

    private fun triggerAction(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().triggerAction(arguments[0] as String?)
        result.success(null)
    }

    private fun triggerActionWithProperties(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().triggerAction(arguments[0] as String?, arguments[1] as HashMap<*, *>?)
        result.success(null)
    }

    private fun triggerActionWithCompletion(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().triggerAction(arguments[0] as String?, object : APMServicesTriggerActionListener {
            override fun success() {
                APMFlutterUtils.resolveResult(null, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }

    private fun triggerActionWithPropertiesAndCompletion(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().triggerAction(arguments[0] as String?, arguments[1] as HashMap<*, *>?, object : APMServicesTriggerActionListener {
            override fun success() {
                APMFlutterUtils.resolveResult(null, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }

    private fun triggerActionWithTag(arguments: ArrayList<*>, result: Result) {
        val tag: APMTag? = APMTag.tagFromHashMap(arguments[0] as HashMap<*, *>?)
        APMPrivate.sharedInstancePrivate().triggerAction(tag, object : APMServicesTriggerActionListener {
            override fun success() {
                APMFlutterUtils.resolveResult(null, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }

    private fun showBadge(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().showBadge()
        result.success(null)
    }

    private fun hideBadge(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().hideBadge()
        result.success(null)
    }

    private fun isOpenBigBadge(arguments: ArrayList<*>, result: Result) {
        result.success(APMPrivate.sharedInstancePrivate().isOpenBigBadge)
    }

    private fun openBigBadge(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().openBigBadge()
        result.success(null)
    }

    private fun openBigBadgeWithController(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().openBigBadgeWithController(arguments[0] as Int)
        result.success(null)
    }

    private fun closeBigBadge(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().closeBigBadge {
            result.success(null)
        }
    }

    private fun showOnboarding(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().showOnboarding()
        result.success(null)
    }

    private fun showOnboardingConnected(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().showOnboardingConnected(arguments[0] as Boolean)
        result.success(null)
    }

    private fun setBaseUrl(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().baseUrl = arguments[0] as String?
        result.success(null)
    }

    private fun getBaseUrl(arguments: ArrayList<*>, result: Result) {
        result.success(APMPrivate.sharedInstancePrivate().baseUrl)
    }

    private fun setDebugMode(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().isDebugMode = arguments[0] as Boolean
        result.success(null)
    }

    private fun isDebugMode(arguments: ArrayList<*>, result: Result) {
        result.success(APMPrivate.sharedInstancePrivate().isDebugMode)
    }

    private fun setDeviceStatus(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().setDeviceStatus(arguments[0] as Int?)
        result.success(null)
    }

    private fun setDeviceOptIn(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().setDeviceOptIn(arguments[0] as Int?)
        result.success(null)
    }

    private fun user(arguments: ArrayList<*>, result: Result) {
        APMFlutterUtils.resolveResult(APMServicesPrivate.sharedInstancePrivate().user, result)
    }

    private fun device(arguments: ArrayList<*>, result: Result) {
        APMFlutterUtils.resolveResult(APMServicesPrivate.sharedInstancePrivate().device, result)
    }

    private fun setBadgeEnabled(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().badgeEnabled = arguments[0] as Boolean
        result.success(null)
    }

    private fun setInterstitialEnabled(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().interstitialEnabled = arguments[0] as Boolean
        result.success(null)
    }

    private fun setNotificationFullscreenEnabled(arguments: ArrayList<*>, result: Result) {
        APMPrivate.sharedInstancePrivate().notificationFullscreenEnabled = arguments[0] as Boolean
        result.success(null)
    }

    //************************************************************************* APMServicesUserListener

    override fun apmServicesUserChanged(user: APMUser?) {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmServicesUserChanged"
        if(user != null) {  result["user"] = APMFlutterUtils.convertObjectToMap(user)!! }
        servicesUserEventSink?.success(result)
    }

    override fun apmServicesUserBalanceChanged(user: APMUser?) {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmServicesUserBalanceChanged"
        if(user != null) {  result["user"] = APMFlutterUtils.convertObjectToMap(user)!! }
        servicesUserEventSink?.success(result)
    }

    override fun apmServicesUserUnsubscribed() {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmServicesUserUnsubscribed"
        servicesUserEventSink?.success(result)
    }

    //************************************************************************* APMOnBadgeClickListener

    override fun onBadgeClick(deeplink: String?): Boolean {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "onBadgeClick"
        if(deeplink != null) {  result["deeplink"] = deeplink }
        onBadgeClickEventSink?.success(result)
        return false
    }
}
