import Flutter
import UIKit
import APM

public class APMModule: NSObject, FlutterPlugin {
    class APMServicesUserStreamHandler: NSObject, FlutterStreamHandler, APMServicesUserListener {
        private var servicesUserEventSink: FlutterEventSink?
        
        func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            APMServices.sharedInstance()!.servicesUserListener = self
            servicesUserEventSink = events
            return nil
        }
        
        func onCancel(withArguments arguments: Any?) -> FlutterError? {
            APMServices.sharedInstance()!.servicesUserListener = nil
            servicesUserEventSink = nil
            return nil
        }
        
        public func apmServicesUserChanged(_ user: APMUser!) {
            var result: [String: Any] = [:]
            result["method"] = "apmServicesUserChanged"
            if(user != nil) { result["user"] = user.toDictionary() }
            self.servicesUserEventSink?(result)
        }
        
        public func apmServicesUserBalanceChanged(_ user: APMUser!) {
            var result: [String: Any] = [:]
            result["method"] = "apmServicesUserBalanceChanged"
            if(user != nil) { result["user"] = user.toDictionary() }
            self.servicesUserEventSink?(result)
        }
        
        public func apmServicesUserUnsubscribed() {
            var result: [String: Any] = [:]
            result["method"] = "apmServicesUserUnsubscribed"
            self.servicesUserEventSink?(result)
        }
    }
    
    class APMOnBadgeClickStreamHandler: NSObject, FlutterStreamHandler {
        private var onBadgeClickEventSink: FlutterEventSink?
        
        func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            APM.sharedInstanceBadgeManager()!.onBadgeClick = { (deeplink: String?) in
                var result: [String: Any] = [:]
                result["method"] = "onBadgeClick"
                if(deeplink != nil) { result["deeplink"] = deeplink }
                self.onBadgeClickEventSink?(result)
                return false
            }
            onBadgeClickEventSink = events
            return nil
        }
        
        func onCancel(withArguments arguments: Any?) -> FlutterError? {
            APM.sharedInstanceBadgeManager()!.onBadgeClick = nil
            onBadgeClickEventSink = nil
            return nil
        }
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = APMModule()
        let channel = FlutterMethodChannel(name: "apm_module", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
        
        let servicesUserEventChannel = FlutterEventChannel(name: "apm_module_services_user_event", binaryMessenger: registrar.messenger())
        servicesUserEventChannel.setStreamHandler(APMServicesUserStreamHandler())
        
        let onBadgeClickEventChannel = FlutterEventChannel(name: "apm_module_on_badge_click_event", binaryMessenger: registrar.messenger())
        onBadgeClickEventChannel.setStreamHandler(APMOnBadgeClickStreamHandler())
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments: [Any] = (call.arguments ?? []) as! [Any]
        
        if (call.method == "initAppWithPartnerID") {
            initAppWithPartnerID(arguments: arguments, result: result)
        }
        else if (call.method == "initAppWithAppID") {
            initAppWithAppID(arguments: arguments, result: result)
        }
        else if (call.method == "refreshSDK") {
            refreshSDK(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActions") {
            triggerActions(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionsWithProperties") {
            triggerActionsWithProperties(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionsWithCompletion") {
            triggerActionsWithCompletion(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionsWithTags") {
            triggerActionsWithTags(arguments: arguments, result: result)
        }
        else if (call.method == "triggerAction") {
            triggerAction(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionWithProperties") {
            triggerActionWithProperties(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionWithCompletion") {
            triggerActionWithCompletion(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionWithPropertiesAndCompletion") {
            triggerActionWithPropertiesAndCompletion(arguments: arguments, result: result)
        }
        else if (call.method == "triggerActionWithTag") {
            triggerActionWithTag(arguments: arguments, result: result)
        }
        else if (call.method == "showBadge") {
            showBadge(arguments: arguments, result: result)
        }
        else if (call.method == "hideBadge") {
            hideBadge(arguments: arguments, result: result)
        }
        else if (call.method == "isOpenBigBadge") {
            isOpenBigBadge(arguments: arguments, result: result)
        }
        else if (call.method == "openBigBadge") {
            openBigBadge(arguments: arguments, result: result)
        }
        else if (call.method == "openBigBadgeWithController") {
            openBigBadgeWithController(arguments: arguments, result: result)
        }
        else if (call.method == "closeBigBadge") {
            closeBigBadge(arguments: arguments, result: result)
        }
        else if (call.method == "showOnboarding") {
            showOnboarding(arguments: arguments, result: result)
        }
        else if (call.method == "showOnboardingConnected") {
            showOnboardingConnected(arguments: arguments, result: result)
        }
        else if (call.method == "setBaseUrl") {
            setBaseUrl(arguments: arguments, result: result)
        }
        else if (call.method == "getBaseUrl") {
            getBaseUrl(arguments: arguments, result: result)
        }
        else if (call.method == "setDebugMode") {
            setDebugMode(arguments: arguments, result: result)
        }
        else if (call.method == "isDebugMode") {
            isDebugMode(arguments: arguments, result: result)
        }
        else if (call.method == "setDeviceStatus") {
            setDeviceStatus(arguments: arguments, result: result)
        }
        else if (call.method == "setDeviceOptIn") {
            setDeviceOptIn(arguments: arguments, result: result)
        }
        else if (call.method == "user") {
            user(arguments: arguments, result: result)
        }
        else if (call.method == "device") {
            device(arguments: arguments, result: result)
        }
        else if (call.method == "setBadgeEnabled") {
            setBadgeEnabled(arguments: arguments, result: result)
        }
        else if (call.method == "setInterstitialEnabled") {
            setInterstitialEnabled(arguments: arguments, result: result)
        }
        else if (call.method == "setNotificationFullscreenEnabled") {
            setNotificationFullscreenEnabled(arguments: arguments, result: result)
        }
        else {
            result(FlutterMethodNotImplemented)
        }
    }

    private func initAppWithPartnerID(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.initApp(withPartnerID: arguments[0] as? String, withPartnerSecret: arguments[1] as? String)
        result(nil)
    }
    
    private func initAppWithAppID(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.initApp(withPartnerID: arguments[0] as? String, withPartnerSecret: arguments[1] as? String, withAppId: arguments[2] as? String)
        result(nil)
    }

    private func refreshSDK(arguments: [Any], result: FlutterResult) {
        APMServices.sharedInstance()!.refreshSDK(arguments[0] as! Bool)
        result(nil)
    }

    private func triggerActions(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.triggerActions(arguments[0] as? [String])
        result(nil)
    }
    
    private func triggerActionsWithProperties(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.triggerActions(arguments[0] as? [String], listProperties: arguments[1] as? [String])
        result(nil)
    }
    
    private func triggerActionsWithCompletion(arguments: [Any], result: @escaping FlutterResult) {
        APM.sharedInstance()!.triggerActions(arguments[0] as? [String], listProperties: arguments[1] as? [String]) { (listStatusActionName: NSMutableArray?, error: Error?) in
            if(error != nil) {
                APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
            }
            else {
                // TODO Vincent: 26/07/21 object: listStatusActionName CRASH !
                APMFlutterUtils.resolveResult(object: nil, result: result)
            }
        }
    }
    
    private func triggerActionsWithTags(arguments: [Any], result: @escaping FlutterResult) {
        APM.sharedInstance()!.triggerActions(withTags: APMTag.tags(fromDictionaries: arguments[0] as? [NSDictionary])) { (listStatusActionName: NSMutableArray?, error: Error?) in
            if(error != nil) {
                APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
            }
            else {
                // TODO Vincent: 26/07/21 object: listStatusActionName CRASH !
                APMFlutterUtils.resolveResult(object: nil, result: result)
            }
        }
    }
    
    private func triggerAction(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.triggerAction(arguments[0] as? String)
        result(nil)
    }
    
    private func triggerActionWithProperties(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.triggerAction(arguments[0] as? String, properties: arguments[1] as? NSMutableDictionary)
        result(nil)
    }
    
    private func triggerActionWithCompletion(arguments: [Any], result: @escaping FlutterResult) {
        APM.sharedInstance()!.triggerAction(arguments[0] as? String) {
            APMFlutterUtils.resolveResult(object: nil, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func triggerActionWithPropertiesAndCompletion(arguments: [Any], result: @escaping FlutterResult) {
        APM.sharedInstance()!.triggerAction(arguments[0] as? String, properties: arguments[1] as? NSMutableDictionary) {
            APMFlutterUtils.resolveResult(object: nil, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func triggerActionWithTag(arguments: [Any], result: @escaping FlutterResult) {
        let tag = APMTag.init(from: arguments[0] as? [AnyHashable : Any])
        APM.sharedInstance()!.triggerAction(with: tag) {
            APMFlutterUtils.resolveResult(object: nil, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func showBadge(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.showBadge()
        result(nil)
    }
    
    private func hideBadge(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.hideBadge()
        result(nil)
    }
    
    private func isOpenBigBadge(arguments: [Any], result: FlutterResult) {
        result(APM.sharedInstance()!.isOpenBigBadge())
    }
    
    private func openBigBadge(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.openBigBadge()
        result(nil)
    }
    
    private func openBigBadgeWithController(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.openBigBadge(withController: arguments[0] as! Int)
        result(nil)
    }
    
    private func closeBigBadge(arguments: [Any], result: @escaping FlutterResult) {
        APM.sharedInstance()!.closeBigBadge {
            result(nil)
        }
    }
    
    private func showOnboarding(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.showOnboarding()
        result(nil)
    }
    
    private func showOnboardingConnected(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.showOnboardingConnected(arguments[0] as! Bool)
        result(nil)
    }
    
    private func setBaseUrl(arguments: [Any], result: FlutterResult) {
        APMServices.sharedInstance()!.baseUrl = arguments[0] as? String
        result(nil)
    }
    
    private func getBaseUrl(arguments: [Any], result: FlutterResult) {
        result(APMServices.sharedInstance()!.baseUrl)
    }
    
    private func setDebugMode(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.setDebugMode(arguments[0] as! Bool)
        result(nil)
    }
    
    private func isDebugMode(arguments: [Any], result: FlutterResult) {
        result(APM.sharedInstance()!.debugMode())
    }
    
    private func setDeviceStatus(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.setDeviceStatus(arguments[0] as? NSNumber)
        result(nil)
    }
    
    private func setDeviceOptIn(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.setDeviceOptIn(arguments[0] as? NSNumber)
        result(nil)
    }
    
    private func user(arguments: [Any], result: FlutterResult) {
        APMFlutterUtils.resolveResult(object: APMServices.sharedInstance()!.user, result: result)
    }
    
    private func device(arguments: [Any], result: FlutterResult) {
        APMFlutterUtils.resolveResult(object: APMServices.sharedInstance()!.device, result: result)
    }
    
    private func setBadgeEnabled(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.badgeEnabled = arguments[0] as! Bool
        result(nil)
    }
    
    private func setInterstitialEnabled(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.interstitialEnabled = arguments[0] as! Bool
        result(nil)
    }
    
    private func setNotificationFullscreenEnabled(arguments: [Any], result: FlutterResult) {
        APM.sharedInstance()!.notificationFullscreenEnabled = arguments[0] as! Bool
        result(nil)
    }
}
