import Flutter
import UIKit
import APM

public class APMUIServicesUserModule: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = APMUIServicesUserModule()
        let channel = FlutterMethodChannel(name: "apm_ui_services_user_module", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments: [Any] = (call.arguments ?? []) as! [Any]
        
        if (call.method == "userClientConnectWithJSON") {
            userClientConnectWithJSON(arguments: arguments, result: result)
        }
        else if (call.method == "userClientConnect") {
            userClientConnect(arguments: arguments, result: result)
        }
        else if (call.method == "userLogout") {
            userLogout(arguments: arguments, result: result)
        }
        else if (call.method == "userDelete") {
            userDelete(arguments: arguments, result: result)
        }
        else {
            result(FlutterMethodNotImplemented)
        }
    }

    private func userClientConnectWithJSON(arguments: [Any], result: @escaping FlutterResult) {
        APMUIServicesUser.userClientConnect(arguments[0] as? [AnyHashable : Any]) { (user: APMUser?) in
            APMFlutterUtils.resolveResult(object: user, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func userClientConnect(arguments: [Any], result: @escaping FlutterResult) {
        let firstname: String? = arguments[0] as? String
        let lastname: String? = arguments[1] as? String
        let email: String? = arguments[2] as? String
        let facebookId: String? = arguments[3] as? String
        let partnerClientId: String? = arguments[4] as? String
        let oldClientId: String? = arguments[5] as? String
        let segments: [AnyHashable: Any]? = arguments[6] as? [AnyHashable: Any]
        let optIn: NSNumber? = arguments[7] as? NSNumber
        
        APMUIServicesUser.userClientConnect(firstname, lastname: lastname, email: email, facebookId: facebookId, partnerClientId: partnerClientId, oldClientId: oldClientId, segments: segments, optIn: optIn) { (user: APMUser?) in
            APMFlutterUtils.resolveResult(object: user, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func userLogout(arguments: [Any], result: @escaping FlutterResult) {
        APMUIServicesUser.userLogout {
            APMFlutterUtils.resolveResult(object: nil, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
    
    private func userDelete(arguments: [Any], result: @escaping FlutterResult) {
        APMUIServicesUser.userDelete {
            APMFlutterUtils.resolveResult(object: nil, result: result)
        } failure: { (error: Error?) in
            APMFlutterUtils.rejectResultWithError(error: error! as NSError, result: result)
        }
    }
}
